unit ss; { *** string services, (c) by ZzSoft *** }
{$I comdir}
interface

function upcase (c:char):char;
function locase (c:char):char;
function string_of (c:char; len:byte):string;
function pos_str (const s1,s2:string; index:byte):byte;
function param_cnt (const s:string; c:char):byte;
function param_str (const s:string; c:char; index:byte):string;
procedure del_char (var s:string; c:char);
procedure del_space (var s:string);
procedure subst_str (var s:string; const substr:string; index,len:integer);

{ --- justify string --- }

function ljust (const s:string; c:char; n:byte):string;
function rjust (const s:string; c:char; n:byte):string;
function cjust (const s:string; c:char; n:byte):string;

{ --- numeric to string conversion --- }

function _str   (x:longint):string;
function _strr  (x:real):string;
function _fstr  (x:longint; w:byte):string;
function _fstrr (x:real; w,d:byte):string;

{ --- string to numeric conversion --- }

function _val   (const s:string; e:pointer):longint;
function _valx  (const s:string; e:pointer):longint;
function _valr  (const s:string; e:pointer):real;

implementation

function upcase (c:char):char; assembler;
{ converts a character to uppercase }
asm
        mov     al,c
        cmp     al,'a'
        jb      @@2
        cmp     al,'z'
        jbe     @@1
        cmp     al,''
        jb      @@2
        cmp     al,''
        jbe     @@1
        cmp     al,''
        jb      @@2
        cmp     al,''
        ja      @@2
        xor     al,50h
 @@1:
        xor     al,20h
 @@2:
end;

function locase (c:char):char; assembler;
{ converts a character to lowercase }
asm
        mov     al,c
        cmp     al,'A'
        jb      @@2
        cmp     al,'Z'
        jbe     @@1
        cmp     al,''
        jb      @@2
        cmp     al,''
        jbe     @@1
        cmp     al,''
        jb      @@2
        cmp     al,''
        ja      @@2
        xor     al,50h
 @@1:
        xor     al,20h
 @@2:
end;

function string_of (c:char; len:byte):string; assembler;
{ returna string of specified character and length }
asm
        les     di,@Result
        mov     al,len
        cld
        stosb
        mov     cl,al
        xor     ch,ch
        mov     al,c
        rep     stosb
end;

function pos_str (const s1,s2:string; index:byte):byte;
{ searches for a s1 in a s2 from specified position }
var
  i:byte;
begin
  i:=pos(s1,copy(s2,index,$FF));
  if i=0 then pos_str:=0 else pos_str:=index+(i-1);
end;

function param_cnt (const s:string; c:char):byte;
{ returns a specified parameter from string in form: 'p1,p2,p3,...' }
var
  i,n:byte;
begin
  if s='' then param_cnt:=0
  else begin
    n:=1;
    for i:=1 to length(s) do if s[i]=c then inc(n);
    if s[length(s)]=c then param_cnt:=n-1 else param_cnt:=n;
  end;
end;

function param_str (const s:string; c:char; index:byte):string;
{ returns a specified parameter from string in form: 'p1,p2,p3,...' }
var
  pos,pos1:integer;
begin
  pos:=1;
  repeat
    pos1:=pos_str(c,s,pos);
    dec(index);
    if (index=0) or (pos1=0) then break;
    pos:=pos1+1;
  until false;
  if index>0 then param_str:=''
  else if pos1<>0 then param_str:=copy(s,pos,pos1-pos)
  else param_str:=copy(s,pos,$FF);
end;

procedure del_char (var s:string; c:char);
{ delete all char c in string }
var
  i:byte;
begin
  for i:=1 to length(s) do if s[i]=c then Delete(s,i,1);
end;

procedure del_space (var s:string);
{ delete all spaces in string }
begin
  del_char(s,' ');
end;

procedure subst_str (var s:string; const substr:string; index,len:integer);
{ substitute string }
begin
  Delete(s,index,len);
  Insert(substr,s,index);
end;

function ljust (const s:string; c:char; n:byte):string;
{ returns left-justified string s in a field of n characters }
begin
  if length(s)<n then ljust:=s+string_of(c,n-length(s))
  else if length(s)>n then ljust:=copy(s,1,n)
  else ljust:=s;
end;

function rjust (const s:string; c:char; n:byte):string;
{ returns left-justified string s in a field of n characters }
begin
  if length(s)<n then rjust:=string_of(c,n-length(s))+s
  else if length(s)>n then rjust:=copy(s,length(s)-n+1,n)
  else rjust:=s;
end;

function cjust (const s:string; c:char; n:byte):string;
{ returns centerted string s in a field of n characters }
begin
  if length(s)<n then cjust:=string_of(c,((n-length(s)) div 2))+s+
                             string_of(c,((n-length(s)+1) div 2))
  else if length(s)>n then cjust:=copy(s,((length(s)-n) div 2)+1,n)
  else cjust:=s;
end;

{ --- numeric to string conversion --- }

function _str (x:longint):string;
var
  s:string[20];
begin
  str(x,s);
  _str:=s;
end;

function _strr (x:real):string;
var
  s:string[20];
begin
  str(x,s);
  _strr:=s;
end;

function _fstr (x:longint; w:byte):string;
var
  s:string;
begin
  str(x:w,s);
  _fstr:=s;
end;

function _fstrr (x:real; w,d:byte):string;
var
  s:string;
begin
  str(x:w:d,s);
  _fstrr:=s;
end;

{ --- string to numeric conversion --- }

function _val (const s:string; e:pointer):longint;
var
  x:longint;
  err:integer;
begin
  val(s,x,err);
  if e<>nil then integer(e^):=err;
  _val:=x;
end;

function _valx (const s:string; e:pointer):longint;
var
  x:longint;
  err:integer;
begin
  val('$'+s,x,err);
  if e<>nil then integer(e^):=err;
  _valx:=x;
end;

function _valr (const s:string; e:pointer):real;
var
  x:real;
  err:integer;
begin
  val(s,x,err);
  if e<>nil then integer(e^):=err;
  _valr:=x;
end;

end.
